package simplegenerator.dto;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.WordUtils;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.ToolFactory;
import org.eclipse.jdt.core.formatter.CodeFormatter;
import org.eclipse.jface.text.Document;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.text.edits.TextEdit;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

public class Generator extends AbstractHandler {

	@Override
	public Object execute(ExecutionEvent event) throws ExecutionException {
		TreeSelection selection = (TreeSelection)HandlerUtil.getCurrentSelection(event);
		
		IWorkbench iworkbench = PlatformUI.getWorkbench();
		IWorkbenchWindow iworkbenchwindow = iworkbench.getActiveWorkbenchWindow();
		IWorkbenchPage iworkbenchpage = iworkbenchwindow.getActivePage();
		IEditorPart ieditorpart = iworkbenchpage.getActiveEditor();
		
		IFile editorInput = ((IFileEditorInput)ieditorpart.getEditorInput()).getFile();
		try{
			if(selection.getFirstElement() instanceof DTOs){
				generiere((DTOs)selection.getFirstElement(), editorInput.getProject());
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		
		return null;
	}

	private void generiere(DTOs dtos, IProject project) throws Exception {
		IFolder folder = project.getFolder("src-gen");
		if(!folder.exists()){
			folder.create(true, true, null);
		}
		
		for (Bean bean : dtos.getBeans()) {
			if(project != null && project.exists() && project.isOpen()){
				generiereDatei(bean, folder, project);
			}
		}
	}

	private void generiereDatei(Bean bean, IFolder folder, IProject project) throws Exception {
			// Package
			IFolder packageFolder = folder.getFolder(bean.getPackage());
			if(!packageFolder.exists()){
				packageFolder.create(true, true, null);
			}
			// File
			IFile file = packageFolder.getFile(bean.getName() + ".java");
			if(file.exists()){
				file.delete(true, null);
			}
			InputStream is = new ByteArrayInputStream(generiereKlasse(bean, project));
			file.create(is, true, null);
	}

	private byte[] generiereKlasse(Bean bean, IProject project) throws Exception {
		StringBuilder sb = new StringBuilder();
		if(!StringUtils.isEmpty(bean.getPackage())){
			sb.append("package ").append(bean.getPackage()).append(";");
		}
		
		sb.append("public class ").append(bean.getName()).append(" {");
		if(!bean.getProperties().isEmpty()){
			// FIELDS
			for (Property prop : bean.getProperties()) {
				sb.append(generiereProperty(prop));
			}
			// METHODS
			for (Property prop : bean.getProperties()) {
				sb.append(generierePropertyMethods(prop));
			}
		}
		sb.append("}");
		
		
		String content = sb.toString();
		
		// TODO improve (do not call for each file)
		IDocument doc = new Document(content);
		Map options = JavaCore.create(project).getOptions(true);
		TextEdit edit = ((CodeFormatter)ToolFactory.createCodeFormatter(options)).format(CodeFormatter.K_COMPILATION_UNIT, doc.get(), 0, doc.getLength(), 0, null);
		if(edit != null){
			edit.apply(doc);
			content = doc.get();
		}
		return content.getBytes();
	}

	

	
	private String generiereProperty(Property prop) {
		StringBuilder sb = new StringBuilder();
		sb.append("private ")
			.append(prop.getType().getLiteral()).append(" ")
			.append(prop.getName()).append(";");
		return sb.toString();
	}
	
	
	private Object generierePropertyMethods(Property prop) {
		StringBuilder sb = new StringBuilder();
		// GETTER
		sb.append(prop.getAccessor().getLiteral()).append(" ").append(prop.getType().getLiteral());
		sb.append(" get").append(WordUtils.capitalize(prop.getName())).append("() { ");
		sb.append("return ").append(prop.getName());
		sb.append(";}");
		// SETTER
		sb.append(prop.getAccessor().getLiteral()).append(" void ");
		sb.append(" set").append(WordUtils.capitalize(prop.getName())).append("(").append(prop.getType().getLiteral()).append(" ").append(prop.getName()).append(") {");
		sb.append("this.").append(prop.getName()).append(" = ").append(prop.getName());
		sb.append(";}");
		return sb.toString();
	}


	@Override
	public boolean isEnabled() {
		return super.isEnabled();
	}
	

}
